@@ -4,6 +4,7 @@ try:
     import http.client
 except ImportError:
     pass
+import logging
 import six
 from six.moves.http_client import (
     HTTPConnection,
@@ -16,26 +17,50 @@ from vcr.request import Request
 from vcr.errors import CannotOverwriteExistingCassetteException
 from . import compat
 
+log = logging.getLogger(__name__)
 
-def parse_headers_backwards_compat(header_dict):
+
+class VCRFakeSocket(object):
     """
-    In vcr 0.6.0, I changed the cassettes to store
-    headers as a list instead of a dict.  This method
-    parses the old dictionary-style headers for
-    backwards-compatability reasons.
+    A socket that doesn't do anything!
+    Used when playing back casssettes, when there
+    is no actual open socket.
     """
-    msg = HTTPMessage(BytesIO(""))
-    for key, val in header_dict.items():
-        msg.addheader(key, val)
-        msg.headers.append("{0}:{1}".format(key, val))
-    return msg
+
+    def close(self):
+        pass
+
+    def settimeout(self, *args, **kwargs):
+        pass
+
+    def fileno(self):
+        """
+        This is kinda crappy.  requests will watch
+        this descriptor and make sure it's not closed.
+        Return file descriptor 0 since that's stdin.
+        """
+        return 0  # wonder how bad this is....
 
 
 def parse_headers(header_list):
-    if isinstance(header_list, dict):
-        return parse_headers_backwards_compat(header_list)
-    headers = b"".join(header_list) + b"\r\n"
-    return compat.get_httpmessage(headers)
+    """
+    Convert headers from our serialized dict with lists for keys to a
+    HTTPMessage
+    """
+    header_string = b""
+    for key, values in header_list.items():
+        for v in values:
+            header_string += \
+                key.encode('utf-8') + b":" + v.encode('utf-8') + b"\r\n"
+    return compat.get_httpmessage(header_string)
+
+
+def serialize_headers(response):
+    out = {}
+    for key, values in compat.get_headers(response):
+        out.setdefault(key, [])
+        out[key].extend(values)
+    return out
 
 
 class VCRHTTPResponse(HTTPResponse):
@@ -94,18 +119,42 @@ class VCRConnection:
     # A reference to the cassette that's currently being patched in
     cassette = None
 
+    def _port_postfix(self):
+        """
+        Returns empty string for the default port and ':port' otherwise
+        """
+        port = self.real_connection.port
+        default_port = {'https': 433, 'http': 80}[self._protocol]
+        return ':{0}'.format(port) if port != default_port else ''
+
+    def _uri(self, url):
+        """Returns request absolute URI"""
+        uri = "{0}://{1}{2}{3}".format(
+            self._protocol,
+            self.real_connection.host,
+            self._port_postfix(),
+            url,
+        )
+        return uri
+
+    def _url(self, uri):
+        """Returns request selector url from absolute URI"""
+        prefix = "{0}://{1}{2}".format(
+            self._protocol,
+            self.real_connection.host,
+            self._port_postfix(),
+        )
+        return uri.replace(prefix, '', 1)
+
     def request(self, method, url, body=None, headers=None):
         '''Persist the request metadata in self._vcr_request'''
-
         self._vcr_request = Request(
-            protocol=self._protocol,
-            host=self.real_connection.host,
-            port=self.real_connection.port,
             method=method,
-            path=url,
+            uri=self._uri(url),
             body=body,
             headers=headers or {}
         )
+        log.debug('Got {0}'.format(self._vcr_request))
 
         # Note: The request may not actually be finished at this point, so
         # I'm not sending the actual request until getresponse().  This
@@ -119,14 +168,12 @@ class VCRConnection:
         of putheader() calls.
         """
         self._vcr_request = Request(
-            protocol=self._protocol,
-            host=self.real_connection.host,
-            port=self.real_connection.port,
             method=method,
-            path=url,
+            uri=self._uri(url),
             body="",
             headers={}
         )
+        log.debug('Got {0}'.format(self._vcr_request))
 
     def putheader(self, header, *values):
         for value in values:
@@ -160,6 +207,11 @@ class VCRConnection:
         if self._vcr_request in self.cassette and \
                 self.cassette.record_mode != "all" and \
                 self.cassette.rewound:
+            log.info(
+                "Playing response for {0} from cassette".format(
+                    self._vcr_request
+                )
+            )
             response = self.cassette.play_response(self._vcr_request)
             return VCRHTTPResponse(response)
         else:
@@ -173,11 +225,16 @@ class VCRConnection:
             # Otherwise, we should send the request, then get the response
             # and return it.
 
+            log.info(
+                "{0} not in cassette, sending to real server".format(
+                    self._vcr_request
+                )
+            )
             self.real_connection.request(
                 method=self._vcr_request.method,
-                url=self._vcr_request.path,
+                url=self._url(self._vcr_request.uri),
                 body=self._vcr_request.body,
-                headers=dict(self._vcr_request.headers or {})
+                headers=self._vcr_request.headers,
             )
 
             # get the response
@@ -189,7 +246,7 @@ class VCRConnection:
                     'code': response.status,
                     'message': response.reason
                 },
-                'headers': compat.get_headers(response),
+                'headers': serialize_headers(response),
                 'body': {'string': response.read()},
             }
             self.cassette.append(self._vcr_request, response)
@@ -220,6 +277,17 @@ class VCRConnection:
 
         return self.real_connection.connect(*args, **kwargs)
 
+    @property
+    def sock(self):
+        if self.real_connection.sock:
+            return self.real_connection.sock
+        return VCRFakeSocket()
+
+    @sock.setter
+    def sock(self, value):
+        if self.real_connection.sock:
+            self.real_connection.sock = value
+
     def __init__(self, *args, **kwargs):
         # need to temporarily reset here because the real connection
         # inherits from the thing that we are mocking out.  Take out
